/*
 * main.c
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/joystick.h>

#define HYST_MIN                ((32768)/3)
#define HYST_MAX                ((32768)/3*2)

char *jsdev = NULL;
int jsfd = 0, n_axis, n_btns;
char *state_axis = NULL;

const char *state_str_btn[] = { "rel", "push" };
const char *state_str_axis[] = { "up", "none", "down" };

int main(int argc, char *argv[]) {
        struct js_event ev;
	int done = 0, b, dir;

	/* args */
	if (argc != 2) {
		fprintf(stderr, "Usage: %s joystick_device\n", argv[0]);
		return -1;
	}
	jsdev = strdup(argv[1]);

	/* joystick init */
	if ((jsfd = open(jsdev, O_RDONLY)) == -1) {
		perror(jsdev);
		return -1;
	}

	/* joystick info */
        if (ioctl(jsfd, JSIOCGAXES, (void *) &n_axis) != 0)
                return -1;
        if (ioctl(jsfd, JSIOCGBUTTONS, (void *) &n_btns) != 0)
                return -1;

	/* alloc state arrays */
	state_axis = (char*) malloc(sizeof(state_axis) * n_axis);
	bzero(state_axis, sizeof(state_axis) * n_axis);

	/* main loop */
	while (!done) {

		/* read event form js */
		b = read(jsfd, &ev, sizeof(ev));
		if (b == 0)
			break;
		if (b == -1 && errno == EINTR)
			continue;

		/* parse event */
		switch(ev.type) {
	                case JS_EVENT_AXIS:
				if (ev.value > HYST_MAX)
					dir = 1;
				if (ev.value < HYST_MIN && ev.value > -HYST_MIN)
					dir = 0;
				if (ev.value < -HYST_MAX)
					dir = -1;

				if (dir != state_axis[ev.number]) {
					printf("axis%d-%s\n", ev.number, state_str_axis[dir + 1]);
					fflush(stdout);
				}

				state_axis[ev.number] = dir;
				break;
				
			case JS_EVENT_BUTTON:
				printf("btn%d-%s\n", ev.number + 1, state_str_btn[ev.value & 1]);
				fflush(stdout);
				break;

			case JS_EVENT_INIT:	/* initial state of device */
				printf("init\n");
				fflush(stdout);
				break;
		}
	
	}
	
	/* clean up */
        close(jsfd);
	return 0;	
}

